package com.ruoyi.common.core.domain.entity;

import java.math.BigDecimal;
import java.util.Date;
import com.fasterxml.jackson.annotation.JsonFormat;
import com.fasterxml.jackson.annotation.JsonView;
import com.ruoyi.common.annotation.Sensitive;
import com.ruoyi.common.core.domain.JsonViewProfile;
import com.ruoyi.common.enums.DesensitizedType;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;
import org.apache.commons.lang3.builder.ToStringBuilder;
import org.apache.commons.lang3.builder.ToStringStyle;
import com.ruoyi.common.annotation.Excel;
import com.ruoyi.common.core.domain.BaseEntity;
import org.springframework.format.annotation.DateTimeFormat;

/**
 * 普通用户信息对象 sm_user
 *
 * @author qzz
 * @date 2024-01-24
 */
@Data
public class SmUser extends BaseEntity
{
    private static final long serialVersionUID = 1L;

    /** 用户ID */
    @ApiModelProperty("用户id")
    @JsonView(JsonViewProfile.App.class)
    private Long userId;

    /** 用户账号 */
    @Excel(name = "用户账号")
    @ApiModelProperty("用户账号")
    @Sensitive(desensitizedType = DesensitizedType.PHONE)
    private String userName;

    /** 用户昵称 */
    @Excel(name = "用户昵称")
    @ApiModelProperty("用户昵称")
    @JsonView(JsonViewProfile.App.class)
    private String nickName;

    /** 用户邮箱 */
    @Excel(name = "用户邮箱")
    @ApiModelProperty("用户邮箱")
    @Sensitive(desensitizedType = DesensitizedType.EMAIL)
    private String email;

    /** 手机号码 */
    @Excel(name = "手机号码")
    @ApiModelProperty("手机号码")
    @Sensitive(desensitizedType = DesensitizedType.PHONE)
    @JsonView(JsonViewProfile.App.class)
    private String phonenumber;

    /** 出生日期 */
    @Excel(name = "出生日期")
    @ApiModelProperty("出生日期")
    private String birthday;

    /** 用户性别 */
    @Excel(name = "用户性别")
    @ApiModelProperty("用户性别，0男，1女，2未知")
    private String sex;

    /** 头像地址 */
    @Excel(name = "头像地址")
    @ApiModelProperty("头像地址")
    @JsonView(JsonViewProfile.App.class)
    private String avatar;

    /** 密码 */
    @Excel(name = "密码")
    @ApiModelProperty("密码")
    @Sensitive(desensitizedType = DesensitizedType.PASSWORD)
    private String password;

    /** 账户余额 */
    @Excel(name = "账户余额")
    @ApiModelProperty("账户余额")
    @JsonView(JsonViewProfile.App.class)
    private BigDecimal balance;

    /** 支付密码 */
    @Excel(name = "支付密码")
    @ApiModelProperty("支付密码")
    @Sensitive(desensitizedType = DesensitizedType.PASSWORD)
    private String payPassword;

    /** 帐号状态 */
    @Excel(name = "帐号状态")
    @ApiModelProperty("帐号状态")
    private String status;

    /** 删除标志 */
    @ApiModelProperty("删除标志")
    private String delFlag;

    /** 最后登录IP */
    @Excel(name = "最后登录IP")
    @ApiModelProperty("最后登录IP")
    private String loginIp;

    /** 最后登录时间 */
    @JsonFormat(pattern = "yyyy-MM-dd")
    @Excel(name = "最后登录时间", width = 30, dateFormat = "yyyy-MM-dd")
    @ApiModelProperty("最后登录时间")
    private Date loginDate;

    /** 微信 */
    @Excel(name = "微信")
    @ApiModelProperty("微信")
    private String wechat;

    /** 身份证号 */
    @ApiModelProperty("身份证号")
    @Sensitive(desensitizedType = DesensitizedType.ID_CARD)
    private String identityCard;

    /** 用户地址 */
    @Excel(name = "用户地址")
    @ApiModelProperty("用户地址")
    private String address ;

    @ApiModelProperty("微信openId")
    private String wxOpenId;

    @ApiModelProperty("逻辑删除时间")
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private Date deleteTime;

    @ApiModelProperty("是否商家")
    @JsonView(JsonViewProfile.App.class)
    private Boolean isMch;
}
