package com.ruoyi.ss.device.mapper;

import com.ruoyi.ss.device.domain.*;
import com.ruoyi.ss.device.domain.vo.SmDeviceVo;
import org.apache.ibatis.annotations.Param;

import java.math.BigDecimal;
import java.util.List;

/**
 * 设备Mapper接口
 *
 * @author 邱贞招
 * @date 2024-02-21
 */
public interface SmDeviceMapper
{
    /**
     * 查询设备
     *
     * @param deviceId 设备主键
     * @return 设备
     */
    public SmDeviceVo selectSmDeviceByDeviceId(Long deviceId);

    /**
     * 查询设备列表
     *
     * @param smDevice 设备
     * @return 设备集合
     */
    public List<SmDeviceVo> selectSmDeviceList(SmDeviceQuery smDevice);

    /**
     * 新增设备
     *
     * @param smDevice 设备
     * @return 结果
     */
    public int insertSmDevice(SmDevice smDevice);

    /**
     * 修改设备
     *
     * @param smDevice 设备
     * @return 结果
     */
    public int updateSmDevice(SmDevice smDevice);

    /**
     * 删除设备
     *
     * @param deviceId 设备主键
     * @return 结果
     */
    public int deleteSmDeviceByDeviceId(Long deviceId);

    /**
     * 批量删除设备
     *
     * @param deviceIds 需要删除的数据主键集合
     * @return 结果
     */
    public int deleteSmDeviceByDeviceIds(Long[] deviceIds);

    /**
     * 条件查询数量
     * @param dto 条件
     */
    int selectCount(SmDeviceQuery dto);

    /**
     * 增加设备时长
     * @param deviceId 设备
     * @param num 增加的量
     */
    int addTime(@Param("deviceId") Long deviceId, @Param("num") BigDecimal num);

    /**
     * 逻辑删除
     */
    int logicDel(@Param("deviceIds") List<Long> deviceIds);

    /**
     * 查询简单数据，不联表
     * @param dto
     * @return
     */
    List<SmDeviceVo> selectSimpleList(SmDeviceQuery dto);

    /**
     * 设备绑定店铺
     * @param deviceId 设备id
     * @param storeId 店铺id
     */
    int bindStore(@Param("deviceId") Long deviceId, @Param("storeId") Long storeId);

    /**
     * 解绑商户
     * @param deviceId 设备id
     */
    int unbindStore(@Param("deviceId") Long deviceId);

    /**
     * 根据MAC查询设备
     * @param mac
     * @return
     */
    SmDeviceVo selectSimpleSmDeviceByMac(String mac);

    /**
     * 根据店铺id列表，查询设备数量
     * @param storeIds 店铺id列表
     */
    List<SmDeviceCountVO> selectCountByStoreIds(@Param("storeIds") List<Long> storeIds);

    /**
     * 条件查询数量
     * @param query 查询条件
     */
    List<SmDeviceCountVO> selectCommonCount(SmDeviceQuery query);

    /**
     * 根据设备编号查询
     * @param deviceNo
     * @return
     */
    SmDeviceVo selectByDeviceNo(String deviceNo);

    /**
     * 设备绑定SN
     */
    int bindSn(@Param("deviceId") Long deviceId, @Param("sn") String sn);

    /**
     * 根据SN查询简单数据
     * @param deviceNo
     * @return
     */
    SmDeviceVo selectSimpleSmDeviceBySn(String deviceNo);
}
