package com.ruoyi.ss.device.service.impl;

import com.ruoyi.common.core.domain.BaseValidator;
import com.ruoyi.common.core.domain.ValidateResult;
import com.ruoyi.common.utils.CollectionUtils;
import com.ruoyi.common.utils.ServiceUtil;
import com.ruoyi.common.utils.UserUtil;
import com.ruoyi.ss.device.domain.SmDeviceQuery;
import com.ruoyi.ss.device.domain.vo.SmDeviceVo;
import com.ruoyi.ss.device.service.DeviceValidator;
import com.ruoyi.ss.device.service.ISmDeviceService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.HashSet;
import java.util.List;
import java.util.Objects;

/**
 * @author wjh
 * 2024/4/30
 */
@Service
public class DeviceValidatorImpl extends BaseValidator implements DeviceValidator {

    @Autowired
    private ISmDeviceService deviceService;

    /**
     * 设备编号是否存在
     *
     * @param deviceNos
     */
    @Override
    public boolean isExistNo(List<String> deviceNos) {
        if (CollectionUtils.isEmptyElement(deviceNos)) {
            return true;
        }
        SmDeviceQuery query = new SmDeviceQuery();
        query.setDeviceNos(deviceNos);
        return deviceService.selectCount(query) == new HashSet<>(deviceNos).size();
    }

    /**
     * 判断设备是否属于指定用户
     *
     * @param deviceId 设备id
     * @param userId   用户id
     */
    @Override
    public boolean isBelong(Long deviceId, Long userId) {
        if (deviceId == null) {
            return false;
        }
        SmDeviceVo device = deviceService.selectSmDeviceByDeviceId(deviceId);
        return device != null && Objects.equals(device.getUserId(), userId);
    }
}
