package com.ruoyi.ss.model.service.impl;

import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.common.utils.ServiceUtil;
import com.ruoyi.ss.device.domain.vo.SmDeviceVo;
import com.ruoyi.ss.device.service.ISmDeviceService;
import com.ruoyi.ss.model.domain.SmModelBO;
import com.ruoyi.ss.model.domain.SmModelQuery;
import com.ruoyi.ss.model.domain.SmModelVo;
import com.ruoyi.ss.model.mapper.SmModelMapper;
import com.ruoyi.ss.model.service.ISmModelService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.util.Collections;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * 型号列表Service业务层处理
 *
 * @author 邱贞招
 * @date 2024-02-21
 */
@Service
public class SmModelServiceImpl implements ISmModelService
{
    @Autowired
    private SmModelMapper smModelMapper;

    @Autowired
    private ISmDeviceService deviceService;

    /**
     * 查询型号列表
     *
     * @param modelId 型号列表主键
     * @return 型号列表
     */
    @Override
    public SmModelVo selectSmModelByModelId(Long modelId)
    {
        return smModelMapper.selectSmModelByModelId(modelId);
    }

    /**
     * 查询型号列表列表
     *
     * @param smModel 型号列表
     * @return 型号列表
     */
    @Override
    public List<SmModelVo> selectSmModelList(SmModelQuery smModel)
    {
        return smModelMapper.selectSmModelList(smModel);
    }

    /**
     * 新增型号列表
     *
     * @param smModel 型号列表
     * @return 结果
     */
    @Override
    public int insertSmModel(SmModelBO smModel)
    {
        smModel.setCreateTime(DateUtils.getNowDate());
        return smModelMapper.insertSmModel(smModel);
    }

    /**
     * 修改型号列表
     *
     * @param smModel 型号列表
     * @return 结果
     */
    @Override
    public int updateSmModel(SmModelBO smModel)
    {
        smModel.setUpdateTime(DateUtils.getNowDate());
        return smModelMapper.updateSmModel(smModel);
    }

    /**
     * 批量删除型号列表
     *
     * @param modelIds 需要删除的型号列表主键
     * @return 结果
     */
    @Override
    public int deleteSmModelByModelIds(Long[] modelIds)
    {
        return smModelMapper.deleteSmModelByModelIds(modelIds);
    }

    /**
     * 删除型号列表信息
     *
     * @param modelId 型号列表主键
     * @return 结果
     */
    @Override
    public int deleteSmModelByModelId(Long modelId)
    {
        return smModelMapper.deleteSmModelByModelId(modelId);
    }

    @Override
    public Integer selectCount(SmModelQuery dto) {
        return smModelMapper.selectCount(dto);
    }

    @Override
    public int logicDelete(List<Long> modelIds) {
        this.validatePreLogicDelete(modelIds);
        return smModelMapper.logicDeleteByIds(modelIds);
    }

    /**
     * 逻辑删除前校验
     */
    private void validatePreLogicDelete(List<Long> modelIds) {
        ServiceUtil.assertion(CollectionUtils.isEmpty(modelIds), "型号列表不允许为空");

        Map<Long, SmModelVo> modelMap = this.selectMapByModelIds(modelIds);
        for (Long modelId : modelIds) {
            SmModelVo model = modelMap.get(modelId);
            ServiceUtil.assertion(model == null || model.getDeleted(), String.format("型号%s不存在", modelId));
        }

        Map<Long, List<SmDeviceVo>> deviceGroup = deviceService.selectGroupByModelIds(modelIds);
        for (Long modelId : modelIds) {
            List<SmDeviceVo> deviceList = deviceGroup.get(modelId);
            ServiceUtil.assertion(!CollectionUtils.isEmpty(deviceList), String.format("型号%s下存在设备，无法删除", modelId));
        }
    }

    /**
     * 根据型号id列表查询数据，并通过map映射
     */
    private Map<Long, SmModelVo> selectMapByModelIds(List<Long> modelIds) {
        if (CollectionUtils.isEmpty(modelIds)) {
            return Collections.emptyMap();
        }
        SmModelQuery dto = new SmModelQuery();
        dto.setModelIds(modelIds);
        List<SmModelVo> list = smModelMapper.selectSmModelList(dto);
        if (CollectionUtils.isEmpty(list)) {
            return Collections.emptyMap();
        }
        return list.stream().collect(Collectors.toMap(SmModelVo::getModelId, item -> item));
    }

}
