package com.ruoyi.ss.suit.domain;

import java.math.BigDecimal;

import com.fasterxml.jackson.annotation.JsonIgnore;
import com.fasterxml.jackson.annotation.JsonView;
import com.ruoyi.common.core.domain.JsonViewProfile;
import com.ruoyi.common.core.domain.ValidGroup;
import com.ruoyi.ss.device.domain.DeviceView;
import lombok.Data;
import lombok.EqualsAndHashCode;
import org.apache.commons.lang3.builder.ToStringBuilder;
import org.apache.commons.lang3.builder.ToStringStyle;
import com.ruoyi.common.annotation.Excel;
import com.ruoyi.common.core.domain.BaseEntity;

import javax.validation.constraints.Min;
import javax.validation.constraints.NotNull;
import javax.validation.constraints.Size;

/**
 * 套餐对象 sm_suit
 *
 * @author ruoyi
 * @date 2024-04-25
 */
@Data
public class Suit extends BaseEntity
{
    private static final long serialVersionUID = 1L;

    /** 套餐id */
    @JsonView({DeviceView.SuitList.class, JsonViewProfile.AppMch.class})
    @NotNull(message = "套餐id不允许为空", groups = {ValidGroup.Update.class, ValidGroup.FrontUpdate.class})
    private Long suitId;

    /** 设备id */
    @Excel(name = "设备id")
    @NotNull(message = "设备ID不允许为空", groups = {ValidGroup.Create.class, ValidGroup.FrontCreate.class})
    @JsonView(JsonViewProfile.AppMch.class)
    private Long deviceId;

    /** 套餐名称 */
    @Excel(name = "套餐名称")
    @JsonView({DeviceView.SuitList.class, JsonViewProfile.AppMch.class})
    @NotNull(message = "套餐名称不允许为空", groups = {ValidGroup.Create.class, ValidGroup.FrontCreate.class})
    @Size(min = 1, max = 50, message = "套餐名称长度在1~50个字符之间")
    private String name;

    /** 通电时间（分） */
    @Excel(name = "通电时间（分钟）")
    @JsonView({DeviceView.SuitList.class, JsonViewProfile.AppMch.class})
    @NotNull(message = "通电时间不允许为空", groups = {ValidGroup.Create.class, ValidGroup.FrontCreate.class})
    @Min(value = 1, message = "通电时间不允许小于1分钟")
    private BigDecimal value;

    /** 价格(元） */
    @Excel(name = "价格(元）")
    @JsonView({DeviceView.SuitList.class, JsonViewProfile.AppMch.class})
    @NotNull(message = "价格不允许为空", groups = {ValidGroup.Create.class, ValidGroup.FrontCreate.class})
    @Min(value = 0, message = "价格不允许小于0")
    private BigDecimal price;

    /** 详细说明 */
    @Excel(name = "详细说明")
    @JsonView({DeviceView.SuitList.class, JsonViewProfile.AppMch.class})
    @Size(max = 200, message = "详细说明长度不能超过200个字符")
    private String description;
}
