package com.ruoyi.ss.suit.service.impl;

import java.util.Collections;
import java.util.List;
import java.util.Map;
import java.util.function.Function;
import java.util.stream.Collectors;

import com.ruoyi.common.utils.CollectionUtils;
import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.ss.suit.domain.SuitQuery;
import com.ruoyi.ss.suit.domain.SuitVo;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.ruoyi.ss.suit.mapper.SuitMapper;
import com.ruoyi.ss.suit.domain.Suit;
import com.ruoyi.ss.suit.service.ISuitService;

/**
 * 套餐Service业务层处理
 *
 * @author ruoyi
 * @date 2024-04-25
 */
@Service
public class SuitServiceImpl implements ISuitService
{
    @Autowired
    private SuitMapper suitMapper;

    /**
     * 查询套餐
     *
     * @param suitId 套餐主键
     * @return 套餐
     */
    @Override
    public SuitVo selectSuitBySuitId(Long suitId)
    {
        return suitMapper.selectSuitBySuitId(suitId);
    }

    /**
     * 查询套餐列表
     *
     * @param suit 套餐
     * @return 套餐
     */
    @Override
    public List<SuitVo> selectSuitList(SuitQuery suit)
    {
        return suitMapper.selectSuitList(suit);
    }

    /**
     * 新增套餐
     *
     * @param suit 套餐
     * @return 结果
     */
    @Override
    public int insertSuit(Suit suit)
    {
        suit.setCreateTime(DateUtils.getNowDate());
        return suitMapper.insertSuit(suit);
    }

    /**
     * 修改套餐
     *
     * @param suit 套餐
     * @return 结果
     */
    @Override
    public int updateSuit(Suit suit)
    {
        suit.setUpdateTime(DateUtils.getNowDate());
        return suitMapper.updateSuit(suit);
    }

    /**
     * 批量删除套餐
     *
     * @param suitIds 需要删除的套餐主键
     * @return 结果
     */
    @Override
    public int deleteSuitBySuitIds(Long[] suitIds)
    {
        return suitMapper.deleteSuitBySuitIds(suitIds);
    }

    /**
     * 删除套餐信息
     *
     * @param suitId 套餐主键
     * @return 结果
     */
    @Override
    public int deleteSuitBySuitId(Long suitId)
    {
        return suitMapper.deleteSuitBySuitId(suitId);
    }

    /**
     * 批量逻辑删除
     *
     * @param suitIds
     * @return
     */
    @Override
    public int logicDel(List<Long> suitIds) {
        return suitMapper.logicDel(suitIds);
    }

    /**
     * 通用查询映射方法
     * @param query 查询条件
     * @param keyMapper 映射方法,需要保证唯一性
     */
    @Override
    public <K> Map<K, SuitVo> selectMap(SuitQuery query, Function<? super SuitVo, ? extends K> keyMapper) {
        List<SuitVo> list = selectSuitList(query);
        if (CollectionUtils.isEmptyElement(list)) {
            return Collections.emptyMap();
        }
        return list.stream().collect(Collectors.toMap(keyMapper, item -> item));
    }

    /**
     * 通用分组方法
     *
     * @param query     查询条件
     * @param keyMapper 分组映射方法，需要保证唯一性
     */
    @Override
    public <K> Map<K, List<SuitVo>> selectGroup(SuitQuery query, Function<? super SuitVo, ? extends K> keyMapper) {
        List<SuitVo> list = selectSuitList(query);
        if (CollectionUtils.isEmptyElement(list)) {
            return Collections.emptyMap();
        }
        return list.stream().collect(Collectors.groupingBy(keyMapper));
    }

    /**
     * 根据设备查询套餐列表
     *
     * @param deviceId
     * @return
     */
    @Override
    public List<SuitVo> selectByDeviceId(Long deviceId) {
        if (deviceId == null) {
            return Collections.emptyList();
        }
        SuitQuery query = new SuitQuery();
        query.setDeviceId(deviceId);
        return this.selectSuitList(query);
    }

    /**
     * 查询套餐数量
     *
     * @param query
     * @return
     */
    @Override
    public int selectCount(SuitQuery query) {
        return suitMapper.selectCount(query);
    }
}
