package com.ruoyi.ss.transactionBill.domain;

import com.fasterxml.jackson.annotation.JsonFormat;
import com.fasterxml.jackson.annotation.JsonView;
import com.ruoyi.common.annotation.Excel;
import com.ruoyi.common.core.domain.BaseEntity;
import com.ruoyi.common.core.domain.JsonViewProfile;
import com.ruoyi.common.core.domain.ValidGroup;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;

import javax.validation.constraints.Min;
import javax.validation.constraints.NotNull;
import java.math.BigDecimal;
import java.util.Date;
import java.util.List;

/**
 * 充值记录对象 sm_transaction_bill
 *
 * @author 邱贞招
 * @date 2024-02-21
 */
@Data
public class TransactionBill extends BaseEntity
{
    private static final long serialVersionUID = 1L;

    /** 主键 */
    @ApiModelProperty("主键")
    @JsonView(JsonViewProfile.Base.class)
    private Long billId;

    /** 订单编号 */
    @ApiModelProperty("订单编号")
    @JsonView(JsonViewProfile.Base.class)
    private String billNo;

    /** 用户 */
    @Excel(name = "用户")
    @ApiModelProperty("用户id")
    private Long userId;

    /** 类型 */
    @ApiModelProperty("类型：1充值，2提现")
    @JsonView(JsonViewProfile.AppMch.class)
    private String type;

    /** 设备 */
    @Excel(name = "设备")
    @ApiModelProperty("设备id")
    @JsonView(JsonViewProfile.Base.class)
    private Long deviceId;

    @ApiModelProperty("设备编号")
    @NotNull(message = "设备编号不允许为空", groups = {ValidGroup.Recharge.class})
    @JsonView(JsonViewProfile.App.class)
    private String deviceNo;

    /** 商户(到账用户) */
    @Excel(name = "商户(到账用户)")
    @ApiModelProperty("商户(到账用户)id")
    @JsonView(JsonViewProfile.AppMch.class)
    private Long mchId;

    /** 交易金额 */
    @Excel(name = "交易金额")
    @ApiModelProperty("交易金额")
    @NotNull(message = "交易金额不允许为空", groups = {ValidGroup.Recharge.class})
    @Min(value = 0, message = "交易金额不允许小于0", groups = {ValidGroup.Recharge.class})
    @Min(value = 20, message = "申请提现金额不允许小于20", groups = {ValidGroup.Withdraw.class})
    @JsonView(JsonViewProfile.App.class)
    private BigDecimal money;

    /** 实际到账金额 */
    @Excel(name = "商户实际到账金额")
    @ApiModelProperty("商户实际到账金额")
    @JsonView(JsonViewProfile.AppMch.class)
    private BigDecimal arrivalAmount;

    /** 手续费 */
    @Excel(name = "手续费")
    @ApiModelProperty("手续费")
    @Min(value = 0, message = "手续费不允许小于0", groups = { ValidGroup.Withdraw.class})
    private BigDecimal serviceCharge;

    @ApiModelProperty("订单状态：1未支付，2支付成功，3已退款，4已取消")
    @JsonView(JsonViewProfile.App.class)
    private String status;

    @ApiModelProperty("渠道id")
    private Long channelId;

    @ApiModelProperty("到账后余额")
    @JsonView(JsonViewProfile.AppMch.class)
    private BigDecimal afterBalance;

    @ApiModelProperty("支付成功的时间")
    private Date payTime;

    @ApiModelProperty("支付过期时间")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private Date expireTime;

    @ApiModelProperty("交易账号（银行卡号、微信openId）")
    @JsonView(JsonViewProfile.AppMch.class)
    private String accountNo;

    @ApiModelProperty("已支付/提现金额")
    private BigDecimal payedAmount;

    @ApiModelProperty("微信商家提现明细id列表")
    private List<String> transferIds;

    @ApiModelProperty("渠道成本")
    private BigDecimal channelCost;

    @ApiModelProperty("设备充值状态，0未充值，1充值成功")
    private String deviceRechargeStatus;

    @ApiModelProperty("套餐id")
    @NotNull(message = "套餐不允许为空", groups = {ValidGroup.Recharge.class})
    @JsonView(JsonViewProfile.App.class)
    private Long suitId;

    @ApiModelProperty("套餐时长（分钟）")
    @NotNull(message = "套餐时长不允许为空", groups = {ValidGroup.Recharge.class})
    @JsonView(JsonViewProfile.App.class)
    private BigDecimal suitTime;

    @ApiModelProperty("套餐开始使用时间")
    private Integer suitStartTime;

    @ApiModelProperty("套餐结束使用时间")
    private Integer suitEndTime;

    @ApiModelProperty("套餐到期时间")
    private Integer suitExpireTime;

    @ApiModelProperty("店铺ID")
    @JsonView(JsonViewProfile.App.class)
    private Long storeId;

    @ApiModelProperty("店铺名称")
    @JsonView(JsonViewProfile.App.class)
    private String storeName;

    @ApiModelProperty("店铺地址")
    @JsonView(JsonViewProfile.App.class)
    private String storeAddress;

    @ApiModelProperty("设备名称")
    @JsonView(JsonViewProfile.App.class)
    private String deviceName;

    @ApiModelProperty("套餐名称")
    @JsonView(JsonViewProfile.App.class)
    private String suitName;

}
