package com.ruoyi.ss.transactionBill.mapper;

import com.ruoyi.ss.dashboard.BillCountVo;
import com.ruoyi.ss.transactionBill.domain.TransactionBill;
import com.ruoyi.ss.transactionBill.domain.TransactionBillQuery;
import com.ruoyi.ss.transactionBill.domain.TransactionBillVo;
import org.apache.ibatis.annotations.Param;

import java.util.Date;
import java.util.List;

/**
 * 充值记录Mapper接口
 *
 * @author 邱贞招
 * @date 2024-02-21
 */
public interface TransactionBillMapper
{
    /**
     * 查询充值记录
     *
     * @param billId 充值记录主键
     * @return 充值记录
     */
    public TransactionBillVo selectSmTransactionBillByBillId(Long billId);

    /**
     * 查询充值记录列表
     *
     * @param smTransactionBill 充值记录
     * @return 充值记录集合
     */
    public List<TransactionBillVo> selectSmTransactionBillList(TransactionBillQuery smTransactionBill);

    /**
     * 新增充值记录
     *
     * @param transactionBill 充值记录
     * @return 结果
     */
    public int insertSmTransactionBill(TransactionBill transactionBill);

    /**
     * 修改充值记录
     *
     * @param transactionBill 充值记录
     * @return 结果
     */
    public int updateSmTransactionBill(TransactionBill transactionBill);

    /**
     * 删除充值记录
     *
     * @param billId 充值记录主键
     * @return 结果
     */
    public int deleteSmTransactionBillByBillId(Long billId);

    /**
     * 批量删除充值记录
     *
     * @param billIds 需要删除的数据主键集合
     * @return 结果
     */
    public int deleteSmTransactionBillByBillIds(Long[] billIds);

    /**
     * 获取账单统计数据
     * @param dto 查询条件
     */
    List<BillCountVo> selectCount(TransactionBillQuery dto);

    /**
     * 充值成功
     * @param billId 订单id
     */
    int rechargeSuccess(@Param("billId") Long billId, @Param("payTime") Date payTime);

    /**
     * 通过订单编号查询订单
     * @param billNo 订单编号
     * @return
     */
    TransactionBill selectSmTransactionBillByBillNo(String billNo);

    /**
     * 订单取消
     * @param billNo 订单编号
     * @param status 订单取消后的状态
     * @return 影响的行数
     */
    int cancelRecharge(@Param("billNo") String billNo, @Param("status") String status);

    /**
     * 审核通过
     * @param billId id
     * @param remark 审核意见
     */
    int doPassWithdraw(@Param("billId") Long billId, @Param("remark") String remark);

    /**
     * 拒绝提现
     * @param billId id
     * @param remark 审核意见
     */
    int doRejectWithdraw(@Param("billId") Long billId, @Param("remark") String remark);

    /**
     * 发起提现打款（打款中...）
     * @param billId 提现订单id
     */
    int doPayWithdraw(Long billId);

    /**
     * 提现成功
     * @param billId id
     */
    int withdrawSuccess(Long billId);

    /**
     * 提现失败
     * @param billId id
     */
    int withdrawFailed(Long billId);

    /**
     * 更新状态为支付中
     * @param billId
     * @return
     */
    int paying(Long billId);

    int updateDeviceRechargeStatus(@Param("billId") Long billId, @Param("status") String status);

    /**
     * 修改设备充值状态
     * @param billIds 需要修改的ID
     * @param status 修改后的设备充值状态
     */
    int updateDeviceRechargeStatusByIds(@Param("billIds") List<Long> billIds, @Param("status") String status);
}
