package com.ruoyi.ss.transactionBill.service;

import com.ruoyi.ss.dashboard.BillCountVo;
import com.ruoyi.ss.transactionBill.domain.TransactionBill;
import com.ruoyi.ss.transactionBill.domain.bo.TransactionBillBO;
import com.ruoyi.ss.transactionBill.domain.TransactionBillQuery;
import com.ruoyi.ss.transactionBill.domain.TransactionBillVo;
import com.ruoyi.ss.transactionBill.domain.enums.TransactionBillPayType;
import com.ruoyi.ss.transactionBill.domain.enums.TransactionBillStatus;

import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.concurrent.TimeUnit;
import java.util.function.Function;

/**
 * 充值记录Service接口
 *
 * @author 邱贞招
 * @date 2024-02-21
 */
public interface TransactionBillService
{
    /**
     * 查询充值记录
     *
     * @param billId 充值记录主键
     * @return 充值记录
     */
    public TransactionBillVo selectSmTransactionBillByBillId(Long billId);

    /**
     * 查询充值记录列表
     *
     * @param smTransactionBillQuery 充值记录查询条件
     * @return 充值记录集合
     */
    public List<TransactionBillVo> selectSmTransactionBillList(TransactionBillQuery smTransactionBillQuery);

    /**
     * 新增充值记录
     *
     * @param transactionBill 充值记录
     * @return 结果
     */
    public int insertSmTransactionBill(TransactionBill transactionBill);

    /**
     * 修改充值记录
     *
     * @param transactionBill 充值记录
     * @return 结果
     */
    public int updateSmTransactionBill(TransactionBill transactionBill);

    /**
     * 批量删除充值记录
     *
     * @param billIds 需要删除的充值记录主键集合
     * @return 结果
     */
    public int deleteSmTransactionBillByBillIds(Long[] billIds);

    /**
     * 删除充值记录信息
     *
     * @param billId 充值记录主键
     * @return 结果
     */
    public int deleteSmTransactionBillByBillId(Long billId);

    /**
     * 获取账单统计
     * @param dto
     * @return
     */
    List<BillCountVo> selectCount(TransactionBillQuery dto);

    /**
     * 创建订单
     * @param body 订单信息
     * @return 新增的数据订单编号
     */
    String addOrder(TransactionBill body);

    /**
     * 获取支付结果
     * @param billNo
     * @return
     */
    boolean getPayResult(String billNo);

    /**
     * 充值成功
     * @param billNo 订单编号
     */
    void rechargeSuccess(String billNo, Date payTime);

    /**
     * 取消充值订单
     * @param billNo 订单编号
     * @param status 取消后的状态
     */
    int cancelRecharge(String billNo, TransactionBillStatus status);

    /**
     * 根据订单编号查询数据
     * @param billNo 订单编号
     */
    TransactionBill selectSmTransactionBillByBillNo(String billNo);

    /**
     * 获取未支付的充值订单
     * @return 订单列表
     */
    List<TransactionBillVo> selectNoPayedRecharge();

    /**
     * 提现
     */
    String addWithdraw(TransactionBillBO data);

    /**
     * 提现审核通过
     * @param billId id
     * @param remark 审核意见
     */
    boolean passWithdraw(Long billId, String remark);

    /**
     * 提现审核拒绝
     * @param billId id
     * @param remark 审核意见
     */
    boolean rejectWithdraw(Long billId, String remark);

    /**
     * 提现打款
     * @param billId 提现单id
     */
    boolean payWithdraw(Long billId);

    /**
     * 提现成功
     * @param billId id
     */
    void withdrawSuccess(Long billId);

    /**
     * 提现失败
     * @param billId id
     */
    void withdrawFailed(Long billId);

    /**
     * 更新微信批次明细单号
     * @param billId 提现单号
     * @param transferIds 微信批次明细列表
     */
    boolean updateWxTransferDetailIds(Long billId, List<String> transferIds);

    /**
     * 查询商户统计信息
     * @param dto 查询条件
     * @return 统计信息
     */
    List<BillCountVo> selectLandlordCount(TransactionBillQuery dto);

    /**
     * 刷新订单支付结果
     * @param billIds
     */
    void refreshPayResult(List<Long> billIds);

    /**
     * 支付中
     * @param billId
     */
    void paying(Long billId);

    /**
     * 手动设备充值
     * @param billId
     * @param validate 是否需要校验
     */
    boolean rechargeDevice(Long billId, boolean validate);

    /**
     * 查询设备充值失败列表
     * @param userId
     * @return
     */
    List<TransactionBillVo> selectDeviceRechargeFailList(Long userId);

    /**
     * 查询支付结果，直到过期
     * @param billNo 订单编号
     * @param delay 延迟时间
     * @param timeUnit 时间单位
     */
    void refreshPayResultBeforeExpire(String billNo, int delay, TimeUnit timeUnit);

    /**
     * 查询数量并映射
     * @param query 查询条件
     * @param keyMapper 映射方法
     */
    <K> Map<K, BillCountVo> selectCountMap(TransactionBillQuery query, Function<? super BillCountVo, ? extends K> keyMapper);
}
