package com.ruoyi.ss.user.mapper;

import com.ruoyi.common.core.domain.entity.SmUser;
import com.ruoyi.ss.user.domain.SmUserQuery;
import com.ruoyi.ss.user.domain.SmUserVo;
import org.apache.ibatis.annotations.Param;

import java.math.BigDecimal;
import java.util.List;

/**
 * 普通用户信息Mapper接口
 *
 * @author qzz
 * @date 2024-01-24
 */
public interface SmUserMapper
{
    /**
     * 查询普通用户信息
     *
     * @param userId 普通用户信息主键
     * @return 普通用户信息
     */
    public SmUserVo selectSmUserByUserId(Long userId);

    /**
     * 查询普通用户信息列表
     *
     * @param smUser 普通用户信息
     * @return 普通用户信息集合
     */
    public List<SmUserVo> selectSmUserList(SmUserQuery smUser);

    /**
     * 新增普通用户信息
     *
     * @param smUser 普通用户信息
     * @return 结果
     */
    public int insertSmUser(SmUser smUser);

    /**
     * 修改普通用户信息
     *
     * @param smUser 普通用户信息
     * @return 结果
     */
    public int updateSmUser(SmUser smUser);

    /**
     * 删除普通用户信息
     *
     * @param userId 普通用户信息主键
     * @return 结果
     */
    public int deleteSmUserByUserId(Long userId);

    /**
     * 批量删除普通用户信息
     *
     * @param userIds 需要删除的数据主键集合
     * @return 结果
     */
    public int deleteSmUserByUserIds(Long[] userIds);

    Integer selectCount(SmUserQuery dto);

    int addBalance(@Param("userId") Long userId, @Param("amount") BigDecimal amount);

    SmUserVo selectSmUserByWxOpenId(String openId);

    SmUserVo selectSimpleById(Long userId);

    /**
     * 查询用户余额统计
     * @param dto 查询条件
     */
    BigDecimal selectBalanceCount(SmUserQuery dto);

    /**
     * 查询设备的租户列表
     * @param deviceId 设备列表
     * @return 租户列表
     */
    List<SmUserVo> selectDeviceTenant(@Param("deviceId") Long deviceId);

    /**
     * 逻辑删除
     * @param userIds
     * @return
     */
    boolean logicDelByIds(@Param("userIds") List<Long> userIds);

    /**
     * 扣减余额
     * @param userId 用户id
     * @param amount 金额
     */
    int subtractBalance(@Param("userId") Long userId, @Param("amount") BigDecimal amount);
}
