package com.ruoyi.ss.wxPay.service;

import com.ruoyi.ss.wxPay.domain.enums.TransferScene;
import com.wechat.pay.java.service.payments.jsapi.model.PrepayWithRequestPaymentResponse;
import com.wechat.pay.java.service.payments.model.Transaction;
import com.wechat.pay.java.service.transferbatch.model.InitiateBatchTransferResponse;
import com.wechat.pay.java.service.transferbatch.model.TransferDetailInput;

import javax.servlet.http.HttpServletRequest;
import java.math.BigDecimal;
import java.util.List;

/**
 * 微信支付服务接口
 * @author 辉
 * 2024/3/11
 */
public interface IWxPayService {

    /**
     * 生成微信预支付订单，并签名
     * @param billNo 平台订单编号
     * @return JSAPI调用
     */
    PrepayWithRequestPaymentResponse prepayWithRequestPayment(String billNo);


    /**
     * 关闭订单
     * @param billNo 平台订单编号
     */
    void closeOrder(String billNo);

    /**
     * 通过微信订单id查询订单信息
     * @param prePayId 微信订单id
     * @return 订单信息
     */
    Transaction queryOrderById(String prePayId);

    /**
     * 通过订单编号查询订单信息
     * @param billNo 订单编号
     * @return 订单信息
     */
    Transaction queryOrderByOutTradeNo(String billNo);

    boolean isSuccess(Transaction transaction);

    /**
     * 微信支付通知
     */
    void payNotify(HttpServletRequest request);

    /**
     * 提现打款
     * @param billId 单号
     */
    InitiateBatchTransferResponse payWithdraw(Long billId);

    /**
     * 微信商户支付通知
     */
    void wxTransferNotify(HttpServletRequest request);

    /**
     * 构造给一个用户转账所需的明细列表，将限额分解成多个明细
     * @param totalAmount 转账总金额
     * @param openId 微信用户openId
     * @param transferScene 转账场景
     */
    List<TransferDetailInput> buildTransferDetailList(BigDecimal totalAmount, String openId, TransferScene transferScene);
}
