package com.ruoyi.web.controller.app;

import com.fasterxml.jackson.annotation.JsonView;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.core.domain.JsonViewProfile;
import com.ruoyi.common.core.domain.ValidGroup;
import com.ruoyi.common.core.page.TableDataInfo;
import com.ruoyi.common.utils.ServiceUtil;
import com.ruoyi.common.utils.StringUtils;
import com.ruoyi.ss.device.domain.DeviceView;
import com.ruoyi.ss.device.domain.SmDevice;
import com.ruoyi.ss.device.domain.SmDeviceQuery;
import com.ruoyi.ss.device.service.DeviceAssembler;
import com.ruoyi.ss.device.service.DeviceValidator;
import com.ruoyi.ss.meterReadingRecord.domain.SmMeterReadingRecordQuery;
import com.ruoyi.ss.device.domain.enums.DevicePowerStatus;
import com.ruoyi.ss.device.domain.vo.SmDeviceVo;
import com.ruoyi.ss.device.service.ISmDeviceService;
import com.ruoyi.ss.meterReadingRecord.service.ISmMeterReadingRecordService;
import com.ruoyi.ss.transactionBill.domain.TransactionBillQuery;
import com.ruoyi.ss.transactionBill.domain.enums.TransactionBillStatus;
import com.ruoyi.ss.transactionBill.domain.enums.TransactionBillType;
import com.ruoyi.ss.transactionBill.service.TransactionBillService;
import com.ruoyi.web.core.annotation.MchRequired;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import java.math.BigDecimal;
import java.util.Collections;
import java.util.List;

/**
 * 设备Controller
 * @author 辉
 * 2024/3/5
 */
@Api(tags = "设备")
@RestController
@RequestMapping("/app/device")
public class AppDeviceController extends BaseController {

    @Autowired
    private ISmDeviceService smDeviceService;

    @Autowired
    private ISmMeterReadingRecordService smMeterReadingRecordService;

    @Autowired
    private DeviceAssembler deviceAssembler;

    @Value("${debug}")
    private Boolean debug;

    @Autowired
    private DeviceValidator deviceValidator;

    @Autowired
    private TransactionBillService transactionBillService;

    /**
     * 修改设备
     */
    @MchRequired
    @ApiOperation("商户修改设备")
    @PutMapping
    public AjaxResult edit(@RequestBody @Validated({ValidGroup.FrontUpdate.class}) SmDevice smDevice) {
        SmDevice device = new SmDevice();
        device.setDeviceId(smDevice.getDeviceId());
        device.setStoreId(smDevice.getStoreId());
        device.setRemark(smDevice.getRemark());
        return toAjax(smDeviceService.updateSmDevice(device));
    }

    @MchRequired
    @ApiOperation("查询商户设备列表")
    @GetMapping("/list")
    public TableDataInfo list(SmDeviceQuery smDevice) {
        startPage();
        smDevice.setUserId(getUserId());
        return getDataTable(smDeviceService.selectSmDeviceList(smDevice));
    }

    @ApiOperation("查询店铺设备列表")
    @GetMapping("/listByStore/{storeId}")
    @JsonView(JsonViewProfile.App.class)
    public TableDataInfo listByStore(@PathVariable("storeId") Long storeId, SmDeviceQuery query) {
        startPage();
        query.setStoreId(storeId);
        List<SmDeviceVo> list = smDeviceService.selectSmDeviceList(query);
        deviceAssembler.assembleBusinessTime(list); // 店铺营业时间
        return getDataTable(list);
    }

    /**
     * 获取设备详细信息
     */
    @MchRequired
    @ApiOperation("获取设备详细信息")
    @GetMapping(value = "/{deviceId}")
    public AjaxResult getInfo(@PathVariable("deviceId") Long deviceId) {
        ServiceUtil.assertion(!deviceValidator.isBelong(deviceId, getUserId()), "这不是您的设备");
        SmDeviceVo device = smDeviceService.selectSmDeviceByDeviceId(deviceId);
        List<SmDeviceVo> list = Collections.singletonList(device);
        deviceAssembler.assembleOrderCountInfo(list);   // 订单统计信息
        return success(device);
    }

    @MchRequired
    @ApiOperation("设备通断电")
    @PutMapping("/{deviceId}/changePower")
    public AjaxResult close(@PathVariable @ApiParam("设备id") Long deviceId, String status) {
        DevicePowerStatus powerStatus = DevicePowerStatus.parse(status);
        return success(smDeviceService.changePowerStatus(deviceId, powerStatus ));
    }

    @MchRequired
    @ApiOperation("绑定设备")
    @PutMapping("/bind")
    public AjaxResult bind(@RequestBody SmDevice device) {
        return success(smDeviceService.bind(device.getStoreId(), device.getDeviceNo()));
    }

    @MchRequired
    @ApiOperation("商户解除设备绑定")
    @DeleteMapping("/mch/unbind/{deviceId}")
    public AjaxResult mchUnbind(@PathVariable @ApiParam("设备id") Long deviceId) {
        ServiceUtil.assertion(!deviceValidator.isBelong(deviceId, getUserId()), "这不是您的设备");
        return success(smDeviceService.unbindStore(deviceId));
    }

    @MchRequired
    @ApiOperation("设备电量归零")
    @PutMapping("{deviceId}/reset")
    public AjaxResult reset(@PathVariable @ApiParam("设备id") Long deviceId) {
        return success(smDeviceService.reset(deviceId));
    }

    @MchRequired
    @ApiOperation("获取设备用电量分析")
    @GetMapping("/electric/count")
    public AjaxResult count(@Validated({ValidGroup.Query.class}) SmMeterReadingRecordQuery dto)
    {
        return success(smMeterReadingRecordService.selectCount(dto));
    }

    @MchRequired
    @ApiOperation("设备充值时长")
    @PutMapping("/addTime/{deviceId}")
    public AjaxResult addTime(@PathVariable @ApiParam("设备id") Long deviceId, @ApiParam("电量（度）") BigDecimal amount)
    {
        ServiceUtil.assertion(!deviceValidator.isBelong(deviceId, getUserId()), "这不是您的设备");
        SmDeviceVo device = smDeviceService.selectSmDeviceByDeviceId(deviceId);
        ServiceUtil.assertion(device == null || !getUserId().equals(device.getUserId()), "设备不存在或您无权充值");
        smDeviceService.addTime(deviceId, amount);
        return success(true);
    }

    @MchRequired
    @ApiOperation("刷新设备信息")
    @GetMapping("/{deviceId}/refreshIot")
    public AjaxResult syncIot(@PathVariable @ApiParam("设备id") Long deviceId) {
        ServiceUtil.assertion(!deviceValidator.isBelong(deviceId, getUserId()), "这不是您的设备");
        smDeviceService.pullDeviceInfo(Collections.singletonList(deviceId));
        return success();
    }

    @ApiOperation("判断设备是否已经被绑定")
    @GetMapping("/isBind")
    public AjaxResult syncIot(@RequestParam(required = false) @ApiParam("设备mac") String mac,
                              @RequestParam(required = false) @ApiParam("设备sn") String deviceNo) {
        if (StringUtils.hasText(mac)) {
            return success(smDeviceService.isBindByMac(mac));
        } else if (StringUtils.hasText(deviceNo)) {
            return success(smDeviceService.isBindBySn(deviceNo));
        } else {
            return error("至少传入一个MAC号或SN码");
        }
    }

    @ApiOperation("获取设备，包含套餐列表")
    @JsonView(DeviceView.SuitList.class)
    @GetMapping("/{deviceNo}/withSuitList")
    public AjaxResult getDeviceWithSuitList(@PathVariable String deviceNo) {
        return success(smDeviceService.getDeviceWithSuitList(deviceNo));
    }

    @ApiOperation("通过SN码获取设备信息")
    @GetMapping("/{deviceNo}/bySn")
    public AjaxResult getByDeviceNo(@PathVariable @ApiParam("设备编号(SN)") String deviceNo) {
        return success(smDeviceService.selectByDeviceNo(deviceNo));
    }
}
