package com.ruoyi.web.controller.app;

import com.ruoyi.common.annotation.Anonymous;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.ss.transactionBill.service.TransactionBillService;
import com.ruoyi.ss.wxPay.service.IWxPayService;
import com.wechat.pay.java.core.exception.ValidationException;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;

/**
 * @author 辉
 * 2024/3/11
 */
@Api(tags = "支付")
@RestController
@RequestMapping("/app/pay")
public class AppPayController {

    @Autowired
    private IWxPayService wxPayService;

    @Autowired
    private TransactionBillService transactionBillService;

    @ApiOperation("微信支付")
    @GetMapping("/wx/{billNo}")
    public AjaxResult wxPay(@PathVariable @ApiParam("订单编号") String billNo) {
        return AjaxResult.success(wxPayService.prepayWithRequestPayment(billNo));
    }

    // 微信支付结果通知
    @ApiOperation("微信支付通知")
    @PostMapping("/notify/wx")
    @Anonymous
    public ResponseEntity<Boolean> wxPayNotify(HttpServletRequest request) {
        try {
            wxPayService.payNotify(request);
        } catch (ValidationException e) {
            return ResponseEntity.status(HttpStatus.UNAUTHORIZED).body(null);
        } catch (Exception e) {
            return ResponseEntity.status(HttpStatus.INTERNAL_SERVER_ERROR).body(null);
        }
        return ResponseEntity.status(HttpStatus.OK).body(null);
    }

    @ApiOperation("查询支付结果")
    @GetMapping("/result/{billNo}")
    public AjaxResult payResult(@PathVariable @ApiParam("订单编号") String billNo) {
        return AjaxResult.success(transactionBillService.getPayResult(billNo));
    }

    @ApiOperation("微信商户支付通知")
    @PostMapping("/notify/wx/transfer")
    public ResponseEntity<Boolean> wxTransferNotify(HttpServletRequest request) {
        try {
            wxPayService.wxTransferNotify(request);
        } catch (ValidationException e) {
            return ResponseEntity.status(HttpStatus.UNAUTHORIZED).body(null);
        } catch (Exception e) {
            return ResponseEntity.status(HttpStatus.INTERNAL_SERVER_ERROR).body(null);
        }
        return ResponseEntity.status(HttpStatus.OK).body(null);
    }

}
