package com.ruoyi.web.controller.app;

import com.fasterxml.jackson.annotation.JsonView;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.core.domain.JsonViewProfile;
import com.ruoyi.common.core.domain.ValidGroup;
import com.ruoyi.common.core.page.TableDataInfo;
import com.ruoyi.common.utils.ServiceUtil;
import com.ruoyi.ss.device.service.DeviceValidator;
import com.ruoyi.ss.suit.domain.SuitBO;
import com.ruoyi.ss.suit.domain.SuitVo;
import com.ruoyi.ss.suit.service.ISuitService;
import com.ruoyi.ss.suit.service.SuitValidator;
import com.ruoyi.web.core.annotation.MchRequired;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import java.util.Arrays;
import java.util.List;

/**
 * @author wjh
 * 2024/5/8
 */
@RestController
@RequestMapping("/app/suit")
public class AppSuitController extends BaseController {


    @Autowired
    private ISuitService suitService;

    @Autowired
    private SuitValidator suitValidator;

    @Autowired
    private DeviceValidator deviceValidator;

    @ApiOperation("查询设备套餐列表")
    @MchRequired
    @JsonView(JsonViewProfile.AppMch.class)
    @GetMapping("/listByDeviceId/{deviceId}")
    public AjaxResult list(@PathVariable Long deviceId)
    {
        ServiceUtil.assertion(!deviceValidator.isBelong(deviceId, getUserId()), "这不是您的设备，无法查看");
        List<SuitVo> list = suitService.selectByDeviceId(deviceId);
        return success(list);
    }

    @ApiOperation("获取套餐详细信息")
    @JsonView(JsonViewProfile.AppMch.class)
    @MchRequired
    @GetMapping("/{suitId}")
    public AjaxResult getInfo(@PathVariable("suitId") Long suitId) {
        return success(suitService.selectSuitBySuitId(suitId));
    }

    @ApiOperation("新增套餐")
    @PostMapping
    @MchRequired
    public AjaxResult add(@RequestBody @Validated({ValidGroup.FrontCreate.class}) SuitBO suit) {
        suit = suit.filterCreateByApp();
        ServiceUtil.assertion(suitValidator.preCreateByApp(suit));
        return success(suitService.insertSuit(suit));
    }

    @ApiOperation("修改套餐")
    @MchRequired
    @PutMapping
    public AjaxResult edit(@RequestBody @Validated({ValidGroup.FrontUpdate.class}) SuitBO suit)
    {
        suit = suit.filterUpdateByApp();
        ServiceUtil.assertion(suitValidator.preUpdateByApp(suit));
        return success(suitService.updateSuit(suit));
    }

    @MchRequired
    @ApiOperation("删除套餐")
    @DeleteMapping("/{suitIds}")
    public AjaxResult remove(@PathVariable @ApiParam("套餐id列表") Long[] suitIds)
    {
        List<Long> ids = Arrays.asList(suitIds);
        ServiceUtil.assertion(suitValidator.preLogicDelByApp(ids));
        return success(suitService.logicDel(ids) >= 1);
    }

}
