package com.ruoyi.web.controller.app;

import com.fasterxml.jackson.annotation.JsonView;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.core.domain.JsonViewProfile;
import com.ruoyi.common.core.domain.ValidGroup;
import com.ruoyi.common.core.page.TableDataInfo;
import com.ruoyi.common.enums.UserType;
import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.common.utils.ServiceUtil;
import com.ruoyi.ss.device.service.DeviceValidator;
import com.ruoyi.ss.transactionBill.domain.TransactionBill;
import com.ruoyi.ss.transactionBill.domain.TransactionBillQuery;
import com.ruoyi.ss.transactionBill.domain.bo.TransactionBillBO;
import com.ruoyi.ss.transactionBill.domain.enums.TransactionBillGroupBy;
import com.ruoyi.ss.transactionBill.domain.enums.TransactionBillStatus;
import com.ruoyi.ss.transactionBill.domain.enums.TransactionBillType;
import com.ruoyi.ss.transactionBill.domain.TransactionBillVo;
import com.ruoyi.ss.dashboard.BillCountVo;
import com.ruoyi.ss.transactionBill.service.TransactionBillService;
import com.ruoyi.ss.transactionBill.service.TransactionAssembler;
import com.ruoyi.ss.transactionBill.service.TransactionBillValidator;
import com.ruoyi.web.core.annotation.MchRequired;
import com.ruoyi.web.core.annotation.UserTypePermission;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.CollectionUtils;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import java.math.BigDecimal;
import java.util.Collections;
import java.util.Date;
import java.util.List;
import java.util.Objects;

/**
 * 充值记录Controller
 *
 * @author 邱贞招
 * @date 2024-02-21
 */
@Api(tags = "充值/提现记录")
@RestController
@RequestMapping("/app/bill")
public class AppTransactionBillController extends BaseController
{
    @Autowired
    private TransactionBillService smTransactionBillService;

    @Autowired
    private TransactionAssembler transactionAssembler;

    @Autowired
    private TransactionBillValidator transactionBillValidator;

    @Autowired
    private DeviceValidator deviceValidator;

    /**
     * 查询充值记录列表
     */
    @ApiOperation("查询本人订单列表")
    @GetMapping("/rechargeList")
    @JsonView(JsonViewProfile.App.class)
    public TableDataInfo rechargeList(TransactionBillQuery smTransactionBill)
    {
        smTransactionBill.setUserId(getUserId());
        smTransactionBill.setType(TransactionBillType.RECHARGE.getType());
        startPage();
        List<TransactionBillVo> list = smTransactionBillService.selectSmTransactionBillList(smTransactionBill);
        transactionAssembler.assembleChannelName(list);
        return getDataTable(list);
    }

    /**
     * 查询提现记录列表
     */
    @ApiOperation("查询本人提现记录列表")
    @GetMapping("/withdrawList")
    public TableDataInfo withdrawList(TransactionBillQuery smTransactionBill)
    {
        startPage();
        smTransactionBill.setUserId(getUserId());
        smTransactionBill.setType(TransactionBillType.WITHDRAW.getType());
        List<TransactionBillVo> list = smTransactionBillService.selectSmTransactionBillList(smTransactionBill);
        return getDataTable(list);
    }

    /**
     * 查询收支列表
     */
    @MchRequired
    @ApiOperation("查询本人收支列表")
    @GetMapping("/list")
    @JsonView(JsonViewProfile.AppMch.class)
    public TableDataInfo list(TransactionBillQuery smTransactionBill)
    {
        startPage();
        smTransactionBill.setUserId(getUserId());
        List<TransactionBillVo> list = smTransactionBillService.selectSmTransactionBillList(smTransactionBill);
        transactionAssembler.assembleChannelName(list);
        return getDataTable(list);
    }

    @ApiOperation("获取订单详情")
    @GetMapping("/{billId}")
    @JsonView(JsonViewProfile.AppUser.class)
    public AjaxResult detail(@PathVariable Long billId) {
        ServiceUtil.assertion(transactionBillValidator.preGetDetailByApp(billId, getUserId()));
        TransactionBillVo bill = smTransactionBillService.selectSmTransactionBillByBillId(billId);
        transactionAssembler.assembleChannelName(Collections.singletonList(bill));
        return AjaxResult.success(bill);
    }

    @MchRequired
    @ApiOperation("获取商户统计信息")
    @JsonView(JsonViewProfile.AppMch.class)
    @GetMapping("/landlordCount")
    public AjaxResult landlordCount(@Validated({ValidGroup.Query.class}) TransactionBillQuery dto) {
        Date now = new Date();
        if (dto.getEndDate() == null) {
            dto.setEndDate(now);
        }
        // 如果按日查询，查询的月份是当前月份，并且当前日期小于7日，则查询最近7天的数据，否则查询当月第一天开始的数据
        if (Objects.equals(dto.getGroupBy(), TransactionBillGroupBy.create_date.name())) {
            if (Objects.equals(dto.getYear(), DateUtils.getYear(now))
                    && Objects.equals(dto.getMonth(), DateUtils.getMonth(now))
                    && DateUtils.getDay(now) < 7) {
                // 七天前
                dto.setStartDate(DateUtils.addDays(dto.getEndDate(), -6));
                dto.setYear(null);
                dto.setMonth(null);
            } else {
                Date firstDay = new Date(dto.getEndDate().getTime());
                firstDay.setYear(dto.getYear() - 1900);
                firstDay.setMonth(dto.getMonth() - 1);
                firstDay.setDate(1);
                dto.setStartDate(firstDay);
            }
        }
        dto.setMchId(getUserId());
        dto.setStatus(TransactionBillStatus.SUCCESS.getStatus());
        AjaxResult ajax = AjaxResult.success(smTransactionBillService.selectLandlordCount(dto));

        // 总收入
        TransactionBillQuery allQuery = new TransactionBillQuery();
        allQuery.setMchId(getUserId());
        List<BillCountVo> totalList = smTransactionBillService.selectCount(allQuery);
        if (CollectionUtils.isEmpty(totalList)) {
            ajax.put("totalRecharge", 0);
        } else {
            ajax.put("totalRecharge", totalList.get(0).getRecharge());
        }

        return ajax;
    }

    @ApiOperation("创建订单")
    @PostMapping("/recharge")
    public AjaxResult addRecharge(@RequestBody @Validated({ValidGroup.Recharge.class}) TransactionBill body) {
        body.setUserId(getUserId());
        body.setChannelId(1L);  // 微信支付
        return AjaxResult.success("操作成功", smTransactionBillService.addOrder(body));
    }

    @ApiOperation("取消充值订单")
    @PutMapping("/recharge/cancel/{billNo}")
    public AjaxResult cancelRecharge(@PathVariable @ApiParam("订单编号") String billNo) {
        return AjaxResult.success(smTransactionBillService.cancelRecharge(billNo, TransactionBillStatus.CANCELED));
    }

    @MchRequired
    @ApiOperation("提现申请")
    @PostMapping("/withdraw")
    public AjaxResult withdraw(@RequestBody @Validated({ValidGroup.Withdraw.class}) TransactionBillBO data) {
        data.setUserId(getUserId());
        return AjaxResult.success("操作成功", smTransactionBillService.addWithdraw(data));
    }

    @ApiOperation("查询设备充值失败列表")
    @GetMapping("/recharge/device/fail/list")
    public AjaxResult selectDeviceRechargeFailList() {
        return AjaxResult.success(smTransactionBillService.selectDeviceRechargeFailList(getUserId()));
    }

    @MchRequired
    @ApiOperation("获取设备充值列表")
    @GetMapping("/rechargeListByDevice")
    @JsonView(JsonViewProfile.AppMch.class)
    public TableDataInfo getDeviceRechargeList(@RequestParam @ApiParam("设备id") Long deviceId) {
        ServiceUtil.assertion(!deviceValidator.isBelong(deviceId, getUserId()), "这不是您的设备，无法查询");
        TransactionBillQuery query = new TransactionBillQuery();
        query.setDeviceId(deviceId);
        query.setType(TransactionBillType.RECHARGE.getType());
        query.setStatus(TransactionBillStatus.SUCCESS.getStatus());
        startPage();
        return getDataTable(smTransactionBillService.selectSmTransactionBillList(query));
    }

}
